@@ -13,22 +13,22 @@
 #
 # ----------------------------------------------------------------------
 
-from apparmor.regex import RE_PROFILE_CAP
-from apparmor.common import AppArmorBug, AppArmorException, type_is_str
-from apparmor.rule import BaseRule, BaseRuleset, logprof_value_or_all, parse_modifiers
 import re
 
-# setup module translations
+from apparmor.common import AppArmorBug, AppArmorException
+from apparmor.regex import RE_PROFILE_CAP
+from apparmor.rule import BaseRule, BaseRuleset, logprof_value_or_all, parse_modifiers
 from apparmor.translations import init_translation
+
 _ = init_translation()
 
 
 class CapabilityRule(BaseRule):
-    '''Class to handle and store a single capability rule'''
+    """Class to handle and store a single capability rule"""
 
     # Nothing external should reference this class, all external users
     # should reference the class field CapabilityRule.ALL
-    class __CapabilityAll(object):
+    class __CapabilityAll:
         pass
 
     ALL = __CapabilityAll
@@ -38,10 +38,8 @@ class CapabilityRule(BaseRule):
     def __init__(self, cap_list, audit=False, deny=False, allow_keyword=False,
                  comment='', log_event=None):
 
-        super(CapabilityRule, self).__init__(audit=audit, deny=deny,
-                                             allow_keyword=allow_keyword,
-                                             comment=comment,
-                                             log_event=log_event)
+        super().__init__(audit=audit, deny=deny, allow_keyword=allow_keyword,
+                         comment=comment, log_event=log_event)
         # Because we support having multiple caps in one rule,
         # initializer needs to accept a list of caps.
         self.all_caps = False
@@ -49,16 +47,16 @@ class CapabilityRule(BaseRule):
             self.all_caps = True
             self.capability = set()
         else:
-            if type_is_str(cap_list):
+            if type(cap_list) is str:
                 self.capability = {cap_list}
-            elif type(cap_list) == list and len(cap_list) > 0:
+            elif type(cap_list) == list and cap_list:
                 self.capability = set(cap_list)
             else:
                 raise AppArmorBug('Passed unknown object to CapabilityRule: %s' % str(cap_list))
             # make sure none of the cap_list arguments are blank, in
             # case we decide to return one cap per output line
             for cap in self.capability:
-                if len(cap.strip()) == 0:
+                if not cap.strip():
                     raise AppArmorBug('Passed empty capability to CapabilityRule: %s' % str(cap_list))
 
     @classmethod
@@ -67,7 +65,7 @@ class CapabilityRule(BaseRule):
 
     @classmethod
     def _parse(cls, raw_rule):
-        '''parse raw_rule and return CapabilityRule'''
+        """parse raw_rule and return CapabilityRule"""
 
         matches = cls._match(raw_rule)
         if not matches:
@@ -88,20 +86,20 @@ class CapabilityRule(BaseRule):
                               comment=comment)
 
     def get_clean(self, depth=0):
-        '''return rule (in clean/default formatting)'''
+        """return rule (in clean/default formatting)"""
 
         space = '  ' * depth
         if self.all_caps:
-            return('%s%scapability,%s' % (space, self.modifiers_str(), self.comment))
+            return ('%s%scapability,%s' % (space, self.modifiers_str(), self.comment))
         else:
             caps = ' '.join(self.capability).strip()  # XXX return multiple lines, one for each capability, instead?
             if caps:
-                return('%s%scapability %s,%s' % (space, self.modifiers_str(), ' '.join(sorted(self.capability)), self.comment))
+                return ('%s%scapability %s,%s' % (space, self.modifiers_str(), ' '.join(sorted(self.capability)), self.comment))
             else:
                 raise AppArmorBug("Empty capability rule")
 
     def is_covered_localvars(self, other_rule):
-        '''check if other_rule is covered by this rule object'''
+        """check if other_rule is covered by this rule object"""
 
         if not self._is_covered_list(self.capability, self.all_caps, other_rule.capability, other_rule.all_caps, 'capability'):
             return False
@@ -109,8 +107,8 @@ class CapabilityRule(BaseRule):
         # still here? -> then it is covered
         return True
 
-    def is_equal_localvars(self, rule_obj):
-        '''compare if rule-specific variables are equal'''
+    def is_equal_localvars(self, rule_obj, strict):
+        """compare if rule-specific variables are equal"""
 
         if not type(rule_obj) == CapabilityRule:
             raise AppArmorBug('Passed non-capability rule: %s' % str(rule_obj))
@@ -145,8 +143,8 @@ class CapabilityRule(BaseRule):
 
 
 class CapabilityRuleset(BaseRuleset):
-    '''Class to handle and store a collection of capability rules'''
+    """Class to handle and store a collection of capability rules"""
 
     def get_glob(self, path_or_rule):
-        '''Return the next possible glob. For capability rules, that's always "capability," (all capabilities)'''
+        """Return the next possible glob. For capability rules, that's always "capability," (all capabilities)"""
         return 'capability,'
