@@ -15,33 +15,32 @@
 
 import re
 
+from apparmor.common import AppArmorBug, AppArmorException
 from apparmor.regex import RE_PROFILE_RLIMIT, strip_quotes
-from apparmor.common import AppArmorBug, AppArmorException, type_is_str
 from apparmor.rule import BaseRule, BaseRuleset, parse_comment, quote_if_needed
-
-# setup module translations
 from apparmor.translations import init_translation
+
 _ = init_translation()
 
-rlimit_size     = ['fsize', 'data', 'stack', 'core', 'rss', 'as', 'memlock', 'msgqueue']  # NUMBER ( 'K' | 'M' | 'G' )
-rlimit_number   = ['ofile', 'nofile', 'locks', 'sigpending', 'nproc', 'rtprio']
-rlimit_time     = ['cpu', 'rttime']  # number + time unit (cpu in seconds+, rttime in us+)
-rlimit_nice     = ['nice']  # a number between -20 and 19.
+rlimit_size = ['fsize', 'data', 'stack', 'core', 'rss', 'as', 'memlock', 'msgqueue']  # NUMBER ( 'K' | 'M' | 'G' )
+rlimit_number = ['ofile', 'nofile', 'locks', 'sigpending', 'nproc', 'rtprio']
+rlimit_time = ['cpu', 'rttime']  # number + time unit (cpu in seconds+, rttime in us+)
+rlimit_nice = ['nice']  # a number between -20 and 19.
 
-rlimit_all      = rlimit_size + rlimit_number + rlimit_time + rlimit_nice
+rlimit_all = rlimit_size + rlimit_number + rlimit_time + rlimit_nice
 
-RE_NUMBER_UNIT  = re.compile('^(?P<number>[0-9]+)\s*(?P<unit>[a-zA-Z]*)$')
-RE_NUMBER       = re.compile('^[0-9]+$')
-RE_UNIT_SIZE    = re.compile('^[0-9]+\s*([KMG]B?)?$')
-RE_NICE         = re.compile('^(-20|-[01]?[0-9]|[01]?[0-9])$')
+RE_NUMBER_UNIT = re.compile(r'^(?P<number>[0-9]+)\s*(?P<unit>[a-zA-Z]*)$')
+RE_NUMBER = re.compile('^[0-9]+$')
+RE_UNIT_SIZE = re.compile(r'^[0-9]+\s*([KMG]B?)?$')
+RE_NICE = re.compile('^(-20|-[01]?[0-9]|[01]?[0-9])$')
 
 
 class RlimitRule(BaseRule):
-    '''Class to handle and store a single rlimit rule'''
+    """Class to handle and store a single rlimit rule"""
 
     # Nothing external should reference this class, all external users
     # should reference the class field RlimitRule.ALL
-    class __RlimitAll(object):
+    class __RlimitAll:
         pass
 
     ALL = __RlimitAll
@@ -51,15 +50,13 @@ class RlimitRule(BaseRule):
     def __init__(self, rlimit, value, audit=False, deny=False, allow_keyword=False,
                  comment='', log_event=None):
 
-        super(RlimitRule, self).__init__(audit=audit, deny=deny,
-                                             allow_keyword=allow_keyword,
-                                             comment=comment,
-                                             log_event=log_event)
+        super().__init__(audit=audit, deny=deny, allow_keyword=allow_keyword,
+                         comment=comment, log_event=log_event)
 
         if audit or deny or allow_keyword:
             raise AppArmorBug('The audit, allow or deny keywords are not allowed in rlimit rules.')
 
-        if type_is_str(rlimit):
+        if type(rlimit) is str:
             if rlimit in rlimit_all:
                 self.rlimit = rlimit
             else:
@@ -72,7 +69,7 @@ class RlimitRule(BaseRule):
         self.all_values = False
         if value == RlimitRule.ALL:
             self.all_values = True
-        elif type_is_str(value):
+        elif type(value) is str:
             if not value.strip():
                 raise AppArmorBug('Empty value in rlimit rule')
 
@@ -112,7 +109,7 @@ class RlimitRule(BaseRule):
 
     @classmethod
     def _parse(cls, raw_rule):
-        '''parse raw_rule and return RlimitRule'''
+        """parse raw_rule and return RlimitRule"""
 
         matches = cls._match(raw_rule)
         if not matches:
@@ -133,11 +130,10 @@ class RlimitRule(BaseRule):
         else:
             raise AppArmorException(_("Invalid rlimit rule '%s' - value missing") % raw_rule)  # pragma: no cover - would need breaking the regex
 
-        return RlimitRule(rlimit, value,
-                           comment=comment)
+        return RlimitRule(rlimit, value, comment=comment)
 
     def get_clean(self, depth=0):
-        '''return rule (in clean/default formatting)'''
+        """return rule (in clean/default formatting)"""
 
         space = '  ' * depth
 
@@ -153,12 +149,12 @@ class RlimitRule(BaseRule):
         else:
             raise AppArmorBug('Empty value in rlimit rule')
 
-        return('%s%sset rlimit%s%s,%s' % (space, self.modifiers_str(), rlimit, value, self.comment))
+        return ('%s%sset rlimit%s%s,%s' % (space, self.modifiers_str(), rlimit, value, self.comment))
 
     def size_to_int(self, value):
         number, unit = split_unit(value)
 
-        if unit == '':
+        if not unit:
             pass
         elif unit == 'K' or unit == 'KB':
             number = number * 1024
@@ -174,26 +170,26 @@ class RlimitRule(BaseRule):
     def time_to_int(self, value, default_unit):
         number, unit = split_unit(value)
 
-        if unit == '':
+        if not unit:
             unit = default_unit
 
-        if unit in ['us', 'microsecond', 'microseconds']:
+        if unit in ('us', 'microsecond', 'microseconds'):
             number = number / 1000000.0
             if default_unit == 'seconds':
                 raise AppArmorException(_('Invalid unit in rlimit cpu %s rule') % value)
-        elif unit in ['ms', 'millisecond', 'milliseconds']:
+        elif unit in ('ms', 'millisecond', 'milliseconds'):
             number = number / 1000.0
             if default_unit == 'seconds':
                 raise AppArmorException(_('Invalid unit in rlimit cpu %s rule') % value)
-        elif unit in ['s', 'sec', 'second', 'seconds']: # manpage doesn't list sec
+        elif unit in ('s', 'sec', 'second', 'seconds'):  # manpage doesn't list sec
             pass
-        elif unit in ['min', 'minute', 'minutes']:
+        elif unit in ('min', 'minute', 'minutes'):
             number = number * 60
-        elif unit in ['h', 'hour', 'hours']:
+        elif unit in ('h', 'hour', 'hours'):
             number = number * 60 * 60
-        elif unit in ['d', 'day', 'days']: # manpage doesn't list 'd'
+        elif unit in ('d', 'day', 'days'):  # manpage doesn't list 'd'
             number = number * 60 * 60 * 24
-        elif unit in ['week', 'weeks']:
+        elif unit in ('week', 'weeks'):
             number = number * 60 * 60 * 24 * 7
         else:
             raise AppArmorException('Unknown unit %s in rlimit %s %s' % (unit, self.rlimit, value))
@@ -201,7 +197,7 @@ class RlimitRule(BaseRule):
         return number
 
     def is_covered_localvars(self, other_rule):
-        '''check if other_rule is covered by this rule object'''
+        """check if other_rule is covered by this rule object"""
 
         if not self._is_covered_plain(self.rlimit, False, other_rule.rlimit, False, 'rlimit'):  # rlimit can't be ALL, therefore using False
             return False
@@ -219,7 +215,7 @@ class RlimitRule(BaseRule):
         return True
 
     def is_equal_localvars(self, rule_obj, strict):
-        '''compare if rule-specific variables are equal'''
+        """compare if rule-specific variables are equal"""
 
         if not type(rule_obj) == RlimitRule:
             raise AppArmorBug('Passed non-rlimit rule: %s' % str(rule_obj))
@@ -246,11 +242,12 @@ class RlimitRule(BaseRule):
             _('Value'),  values_txt,
         ]
 
+
 class RlimitRuleset(BaseRuleset):
-    '''Class to handle and store a collection of rlimit rules'''
+    """Class to handle and store a collection of rlimit rules"""
 
     def get_glob(self, path_or_rule):
-        '''Return the next possible glob. For rlimit rules, that can mean changing the value to 'infinity' '''
+        """Return the next possible glob. For rlimit rules, that can mean changing the value to 'infinity'"""
         # XXX implement all options mentioned above ;-)
         raise AppArmorBug('get_glob() is not (yet) available for this rule type')
 
@@ -264,5 +261,3 @@ def split_unit(value):
     unit = matches.group('unit') or ''
 
     return number, unit
-
-
