@@ -51,7 +51,7 @@ class TestBaserule(AATest):
             obj.is_covered_localvars(None)
 
     def test_parse_modifiers_invalid(self):
-        regex = re.compile('^\s*(?P<audit>audit\s+)?(?P<allow>allow\s+|deny\s+|invalid\s+)?')
+        regex = re.compile(r'^\s*(?P<audit>audit\s+)?(?P<allow>allow\s+|deny\s+|invalid\s+)?')
         matches = regex.search('audit invalid ')
 
         with self.assertRaises(AppArmorBug):
