@@ -9,35 +9,41 @@
 #
 # ------------------------------------------------------------------
 
-import apparmor.aa as aa
 import unittest
-from common_test import AAParseTest, setup_regex_tests, setup_aa
+
+import apparmor.aa as aa
+from common_test import AAParseTest, setup_aa, setup_regex_tests
+
 
 class BaseAAParseMountTest(AAParseTest):
     def setUp(self):
         self.parse_function = aa.parse_mount_rule
 
+
 class AAParseMountTest(BaseAAParseMountTest):
-    tests = [
+    tests = (
         ('mount,', 'mount base keyword rule'),
         ('mount -o ro,', 'mount ro rule'),
         ('mount -o rw /dev/sdb1 -> /mnt/external,', 'mount rw with mount point'),
-    ]
+    )
+
 
 class AAParseRemountTest(BaseAAParseMountTest):
-    tests = [
+    tests = (
         ('remount,', 'remount base keyword rule'),
         ('remount -o ro,', 'remount ro rule'),
         ('remount -o ro /,', 'remount ro with mountpoint'),
-    ]
+    )
+
 
 class AAParseUmountTest(BaseAAParseMountTest):
-    tests = [
+    tests = (
         ('umount,', 'umount base keyword rule'),
         ('umount /mnt/external,', 'umount with mount point'),
         ('unmount,', 'unmount base keyword rule'),
         ('unmount /mnt/external,', 'unmount with mount point'),
-    ]
+    )
+
 
 setup_aa(aa)
 if __name__ == '__main__':
